unit Main;

(*______________________________________________________________________

    Project:      "TeleNokia"
    Unit:         "Main"
    Description:  User interface and main function implementation.

                                                                  v00.02
                                                     from:  Jan 08, 2003
                                                            Jan 08, 2003
    http://axofiber.org.ru/mobile/telenokia.php
    Maxim Proskurnya, max@axofiber.org.ru
  ______________________________________________________________________

  Log:
    09.I.2003 - v00.02
        Added noncomplete 'HEX' charset support (needs further testings
        with Nokia 8210). v00.02 still supports only true Unicode models
        (e.g. 8310).
  ______________________________________________________________________
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TNME, RXCtrls, ExtCtrls, Placemnt, StdCtrls, ComCtrls, Buttons,
  Menus, Mask, ToolEdit;

type
  TMainForm = class(TForm)
    ConnectBtn: TRxSpeedButton;
    ExportBtn: TRxSpeedButton;
    Image1: TImage;
    Image2: TImage;
    Image3: TImage;
    PortNumberCB: TComboBox;
    StatusBar: TRxLabel;
    PhoneModelLB: TRxLabel;
    PhoneModel: TRxLabel;
    SaveDialog: TSaveDialog;
    ProgressBar: TProgressBar;
    RxLabel1: TRxLabel;
    procedure QuitBtnClick(Sender: TObject);
    procedure ConnectBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure ExitBtnClick(Sender: TObject);
    procedure ExportBtnClick(Sender: TObject);
    function  Format ( Line: string ): string;
    procedure PortNumberCBKeyPress(Sender: TObject; var Key: Char);
    procedure PortNumberCBChange(Sender: TObject);
    procedure ReadContacts;
  private
    { Private declarations }
  public
    { Public declarations }
    ME      : TTNME;
    Records : TStringList;
  end;

var
  MainForm: TMainForm;

implementation


{$R *.DFM}



  procedure TMainForm.FormCreate(Sender: TObject);
  begin
    ME      := TTNME.Create;
    Records := nil;
  end;



  procedure TMainForm.QuitBtnClick(Sender: TObject);
  begin
    Close;
  end;



  procedure TMainForm.ConnectBtnClick(Sender: TObject);
  var
    Port: string;
    F:    TextFile;
    s:    string;
  begin
    if ME.IsConnected then
    begin
      ME.Stop;
      Image3.Visible        := False;
      ConnectBtn.Caption    := '';
      PortNumberCB.Enabled  := True;
      StatusBar.Caption     := '';
      PhoneModelLB.Visible  := False;
      PhoneModel.Visible    := False;
      ME.Disconnect;
    end
    else
    begin
      Port := PortNumberCB.Text;
      if Port <> '' then
      begin
        try
          StatusBar.Caption     := '   ' + Port;
          ME.Connect ( Port );
          Image3.Visible        := True;
          ConnectBtn.Caption    := '';
          PortNumberCB.Enabled  := False;
          StatusBar.Caption     := ME.StatusMessage;
          PhoneModel.Caption    := ME.Identification;
        except
          on Error: Exception do begin
            StatusBar.Caption   := ME.StatusMessage;
          end;
        end;
        Application.ProcessMessages;
        ReadContacts;
      end;
    end;
  end;



  procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
  begin
    ME.Free;
    if Assigned ( Records ) then Records.Free;
  end;



  procedure TMainForm.ReadContacts;
  var
    i       : Integer;
  begin
    if ME.IsValid then
    begin
      ME.LoadSupportedCharset;
      ProgressBar.Visible   := True;
      Records := ME.ReadContacts ( ProgressBar );
      if Assigned ( Records ) then
      begin
        for i := 0 to Records.Count - 1 do
          Records[i] := Format ( Records[i] );
        StatusBar.Caption  := '  ';
        ExportBtn.Enabled  := True;
      end;
      ProgressBar.Visible := False;
    end;
  end;



  procedure TMainForm.ExitBtnClick(Sender: TObject);
  begin
    Close;
  end;



  procedure TMainForm.PortNumberCBKeyPress(Sender: TObject; var Key: Char);
  begin
    Key := #0;
  end;



  procedure TMainForm.PortNumberCBChange(Sender: TObject);
  begin
    ConnectBtn.Enabled := True;
  end;



  procedure TMainForm.ExportBtnClick(Sender: TObject);
  var
    Book  : TextFile;
    i     : Integer;
  begin
    if SaveDialog.Execute and ( SaveDialog.FileName <> '' ) then
    begin
      AssignFile ( Book, SaveDialog.FileName );
      ReWrite ( Book );
      WriteLn ( Book, 'Name, Phone' );
      if Records <> nil then
        for i := 0 to Records.Count - 1 do
        begin
          WriteLn ( Book, Records[i] );
        end;
      CloseFile ( Book );
    end;
  end;



  function TMainForm.Format ( Line: string ): string;
  var
    p: Integer;
    Name, Phone: string;
  begin
    Phone  := '';
    Name   := '';
    Result := '';
    p := Pos ( ',', Line );
    if p > 0 then
    begin
      Line := Copy ( Line, p + 1, Length ( Line ) - p );
      p := Pos ( ',', Line );
      if p > 0 then
      begin
        Phone := Copy ( Line, 1, p - 1 );
        Line  := Copy ( Line, p + 1, Length ( Line ) - p );
        p := Pos ( ',', Line );
        if p > 0 then
        begin
          Name := Copy ( Line, p + 1, Length ( Line ) - p );
          if ( Name <> '' ) and ( Phone <> '') then
            Result := Name + ',' + Phone;
        end;
      end;
    end;
  end;



  end.
