unit TNME;

(*______________________________________________________________________

    Project:      "TeleNokia"
    Unit:         "TNME"
    Description:  Mobile equipment wrapper implementation

                                                                  v00.01
                                                     from:  Jan 09, 2003
                                                            Jan 09, 2003
    http://axofiber.org.ru/mobile/telenokia.php
    Maxim Proskurnya, max@axofiber.org.ru
  ______________________________________________________________________

  Log:
  ______________________________________________________________________
*)

interface

uses
  TNMEComm, Classes, ComCtrls;

type

  TTNME = class
    MEComm            : TTNMEComm;
    Answers           : TMSAnswer;
    CantContinue      : Boolean;
  public
    Identification    : string;
    Status            : TMEStatus;
    StatusMessage     : string;
    SupportedCharset  : string;

    procedure   Connect ( ACommPort: string );
    procedure   Disconnect;
    function    FromHex ( C: Char ): Integer;
    function    IsConnected: Boolean;
    function    IsHex ( C: Char ): Boolean;
    function    IsValid: Boolean;
    procedure   LoadSupportedCharset;
    function    ReadContacts ( ProgressBar: TProgressBar ): TStringList;
    procedure   Stop;
    function    Translate ( Line: string ): string;

    constructor Create;
    destructor  Destroy; override;
  end;

implementation

uses
  SysUtils;



  constructor TTNME.Create;
  begin
    MEComm        := nil;
    Status        := ME_Valid;
    Answers       := TMSAnswer.Create;
    CantContinue  := False;
  end;



  procedure TTNME.Connect ( ACommPort: string );
  begin
    if IsConnected then
      Disconnect;
    try
      Status := ME_INVALID;
      MEComm := TTNMEComm.Open ( ACommPort );
      MEComm.Write ( 'ATZ' );
      MEComm.Read ( Answers );
      if Answers.Valid and ( Answers.LastLine = 'OK' ) then
      begin
        MEComm.Write ( 'AT+GMM' );
        MEComm.Read ( Answers );
        if Answers.Valid and ( Answers.Count > 2 ) and ( Answers.LastLine = 'OK' ) then
        begin
          Status          := ME_VALID;
          Identification  := Answers[1];
          StatusMessage   := ' ';
        end
        else
          StatusMessage   := '  GSM-';
      end
      else
        StatusMessage     := '   ';
    except
      StatusMessage       := 'C ';
      Disconnect;
      raise;
    end;
  end;




  procedure TTNME.Disconnect;
  begin
    if IsConnected then
    begin
      FreeAndNil ( MEComm );
      Status            := ME_INVALID;
      Identification    := '';
      SupportedCharset  := '';
    end;
  end;



  function TTNME.FromHex ( C: Char ): Integer;
  begin
    Result := -1;
    if ( '0' <= c ) and ( c <= '9' ) then Result := ord ( c ) - 48
    else if ( 'A' <= c ) and ( c <= 'F' ) then Result := ord ( c ) - ord ( 'A' ) + 10
    else if ( 'a' <= c ) and ( c <= 'f' ) then Result := ord ( c ) - ord ( 'a' ) + 10
  end;



  function TTNME.IsConnected: Boolean;
  begin
    Result := MEComm <> nil;
  end;



  function TTNME.IsHex ( C: Char ): Boolean;
  begin
    if ( '0' <= c ) and ( c <= '9' ) or
       ( 'A' <= c ) and ( c <= 'F' ) or
       ( 'a' <= c ) and ( c <= 'f' )
    then
      Result := True
    else
      Result := False;
  end;



  function TTNME.IsValid: Boolean;
  begin
    Result := Status = ME_VALID;
  end;



  procedure TTNME.LoadSupportedCharset;
  begin
    if IsValid then
    begin
      // Check preferable Unicode charset
      MEComm.Write ( 'AT+CSCS="UCS2"' );
      MEComm.Read ( Answers );
      if Answers.Valid and ( Answers.LastLine = 'OK' ) then
        SupportedCharset := 'UCS2'
      else
      begin
        // ..otherwise try hexadecimal representation
        MEComm.Write ( 'AT+CSCS="HEX"' );
        MEComm.Read ( Answers );
        if Answers.Valid and ( Answers.LastLine = 'OK' ) then
          SupportedCharset := 'HEX'
        else
        begin
          Status        := ME_INVALID;
          StatusMessage := '  ';
        end;
      end;
    end;
  end;



  function TTNME.ReadContacts ( ProgressBar: TProgressBar ): TStringList;
  var
    i : Integer;
  begin
    Result := TStringList.Create;
    if IsValid then
    begin
      ProgressBar.Position  := 0;
      ProgressBar.Max       := 1000;
      // Select SIM memory bamk
      MEComm.Write ( 'AT+CPBS="SIM"' );
      MEComm.Read ( Answers );
      if Answers.Valid and ( Answers.LastLine = 'OK' ) then
      begin
        for i := 1 to 500 do
        begin
          if CantContinue then break;
          if i mod 10 = 0 then  ProgressBar.Position := i;
          MEComm.Write ( 'AT+CPBR=' + IntToStr ( i ) );
          MEComm.Read ( Answers );
          if ( Answers.Count > 2 ) then Result.Add ( Translate ( Answers[1] ) );
        end;
      end;

      // Select ME device memory bamk
      ProgressBar.Position := 500;
      MEComm.Write ( 'AT+CPBS="ME"' );
      MEComm.Read ( Answers );
      if Answers.Valid and ( Answers.LastLine = 'OK' ) then
      begin
        for i := 1 to 500 do
        begin
          if CantContinue then break;
          if i mod 10 = 0 then  ProgressBar.Position := i + 500;
          MEComm.Write ( 'AT+CPBR=' + IntToStr ( i ) );
          MEComm.Read ( Answers );
          if ( Answers.Count > 2 ) then Result.Add ( Translate ( Answers[1] ) );
        end;
      end;
    end;
    CantContinue := False;
  end;



  procedure TTNME.Stop;
  begin
    CantContinue := False;
  end;



  function TTNME.Translate ( Line: string ): string;
  var
    i, StartPos, EndPos, len, code: Integer;
    StartPosNotSet, EndPosNotSet : Boolean;
    c : Char;
  begin
    Result          := Line;
    StartPos        := 1;
    EndPos          := Length ( Line );
    EndPosNotSet    := True;
    StartPosNotSet  := True;
    for i := endpos downto 1 do
    begin
      if Line[i] = '"' then
        if EndPosNotSet then
        begin
          EndPos := i - 1;
          EndPosNotSet := False;
        end
        else if StartPosNotSet then
        begin
          StartPos := i + 1;
          StartPosNotSet := False;
        end;
      if ( Line[i] = ',' ) and EndPosNotSet then
      begin
        StartPos        := i + 1;
        StartPosNotSet  := False;
        EndPosNotSet    := False;
      end;
    end;
    if ( 0 < StartPos ) and ( StartPos < EndPos ) then
    begin
      // Check string
      i := StartPos;
      while ( i < EndPos ) and IsHex ( Line[i] ) do
        i := i + 1;
      if IsHex ( Line[i] ) then
      begin
        // Translate hex name
        Result := Copy ( Line, 1, StartPos - 1 );
        i := StartPos;
        if ( SupportedCharset = 'UCS2' ) and ( ( EndPos - StartPos + 1 ) mod 4 = 0 ) then
        begin
          while i < EndPos do
          begin
            code := ( ( ( FromHex ( Line[i] ) * 16 + FromHex ( Line[i+1] ) ) * 16 + FromHex ( Line[i+2] ) ) * 16 ) + FromHex ( Line[i+3] );
            if $0400 < Code then Code := Code - $0350;
            Result := Result + chr ( code );
            i := i + 4;
          end;
          Result := Result + '"';
        end
        else if ( SupportedCharset = 'HEX' ) and ( ( EndPos - StartPos + 1 ) mod 2 = 0 ) then
        begin
          while i < EndPos do
          begin
            code := FromHex ( Line[i] ) * 16 + FromHex ( Line[i+1] );
            case code of
              19 : c := '';
              93 : c := '';
              96 : c := 'a';
              124..130: c := chr ( code + 102 ); // -
              137: c := '';
              138..141: c := chr ( code + 96 ); // -
              143: c := '';
              149: c := '';
              156..158: c := chr ( code + 89 ); // -
              176: c := '';
              205..208: c := chr ( code - 13 ); // -
              218..219: c := chr ( code - 16 ); // -;
              221: c := '';
              225: c := '';
            else
              c := chr ( code );
            end;
            Result := Result + c;
            i := i + 2;
          end;
          Result := Result + '"';
        end
        else
          Result := Line;
      end
      else
        // Error... copy string untranslated
        Result := Line;
    end;
  end;


  destructor TTNME.Destroy;
  begin
    if IsConnected then MEComm.Free;
    Answers.Free;
  end;

end.
