unit TNMEComm;

(*______________________________________________________________________

    Project:      "TeleNokia"
    Unit:         "Communication"
    Description:  Communication with the serial port(COM) implementation

                                                                  v00.02
                                                     from:  Jan 08, 2003
                                                            Jan 09, 2003
    http://axofiber.org.ru/mobile/telenokia.php
    Maxim Proskurnya, max@axofiber.org.ru
  ______________________________________________________________________

  Log:
    09.I.2003 - v00.02
        Fixed line separation block in "Read".
  ______________________________________________________________________
*)


interface

uses
  Windows, Classes, Dialogs;

type


  TMEStatus = ( ME_VALID, ME_INVALID );
  

  TMSAnswer = class
    Lines   : TStringList;
    Status  : TMEStatus;
    function GetLine ( AnIndex: Integer ): string;
  public
    function Count    : Integer;
    function LastLine : string;
    property Items [ AnIndex: Integer ]: string read GetLine; default;
    function Valid: Boolean;
    constructor Create;
    destructor  Destroy; override;
  end;


  TTNMEComm = class
  private
    Buffer    : string[255];
    Input     : string;
    Output    : string;
    CommPort  : string;
    OutLength : Integer;
    TimeOut   : Integer;
    Error     : DWord;
    CommHandle: THandle;

    function    Listen: Boolean;
  public

    function    Read ( Answers: TMSAnswer ): Boolean;
    function    Write ( Data: string ): Boolean;

    constructor Open ( ACommPort: string );
    destructor  Destroy; override;
  end;


implementation

uses
  SysUtils;

//  TMSAnswer

  constructor TMSAnswer.Create;
  begin
    Lines := TStringList.Create;
  end;



  function TMSAnswer.Count: Integer;
  begin
    Result := Lines.Count;
  end;



  function TMSAnswer.GetLine ( AnIndex: Integer ): string;
  begin
    Result := Lines[AnIndex];
  end;



  function TMSAnswer.LastLine: string;
  begin
    if Lines.Count > 0 then
      Result := Lines[Lines.Count - 1]
    else
      Result := '';
  end;



  function TMSAnswer.Valid: Boolean;
  begin
    Result := Status = ME_VALID;
  end;



  destructor TMSAnswer.Destroy;
  begin
    Lines.Free;
  end;


//  TMSComm

  constructor TTNMEComm.Open ( ACommPort: string );
  var
    DCB:        TDCB;
    TimeOuts:   TCommTimeOuts;
    Success :   Boolean;
  begin
    CommPort    := ACommPort;
    CommHandle  := 0;
    CommHandle  := CreateFile ( PChar ( CommPort ),
                     GENERIC_READ or GENERIC_WRITE,
                     0,    // comm devices must be opened w/exclusive-access
                     nil,  // no security attributes
                     OPEN_EXISTING, // comm devices must use OPEN_EXISTING
                     0,    // no overlapped I/O
                     0     // hTemplate must be NULL for comm devices
                     );
    if CommHandle = INVALID_HANDLE_VALUE then
      raise Exception.Create ( 'Can''t create communication.' );

    Success  := GetCommState ( CommHandle, DCB);
    if not Success  then
      raise Exception.Create ( 'Can''t open communication port.' );
    // Fill in the DCB: baud=115,200 bps, 8 data bits, no parity, and 1 stop bit.
    DCB.BaudRate  := CBR_115200;      // set the baud rate
    DCB.ByteSize  := 8;               // data size, xmit, and rcv
    DCB.Parity    := NOPARITY;        // no parity bit
    DCB.StopBits  := ONESTOPBIT;      // one stop bit

    Success       := SetCommState ( CommHandle, DCB );
    if not Success  then
      raise Exception.Create ( 'Can''t configure communication port.' );
    Success       := GetCommTimeOuts ( CommHandle, TimeOuts );
    if not Success  then
      raise Exception.Create ( 'Can''t configure port timeouts.' );
    TimeOuts.ReadIntervalTimeout          := 2;
    TimeOuts.ReadTotalTimeoutMultiplier   := 0;
    TimeOuts.ReadTotalTimeoutConstant     := 0;
    TimeOuts.WriteTotalTimeoutMultiplier  := 0;
    TimeOuts.WriteTotalTimeoutConstant    := 0;

    Success       := SetCommTimeOuts ( CommHandle, TimeOuts );
    if not Success  then
      raise Exception.Create ( 'Can''t configure port timeouts.' );
    sleep ( 2000 );
//ShowMessage ( '1' );
  end;



  function  TTNMEComm.Listen: Boolean;
  var
    Finished          : Boolean;
    i                 : Integer;
    NumberOfBytesRead : DWord;
  begin
    TimeOut   := 10;
    Finished  := False;
    Result    := True;
    if Pos ( #13, Input ) > 0 then
      Input   := '';
    repeat
      ReadFile ( CommHandle, Buffer[1], 255, NumberOfBytesRead, nil );
      if NumberOfBytesRead > 0 then
      begin
        for i := 1 to NumberOfBytesRead do
          Input := Input + Buffer[i];
        Finished := True;
      end
      else
      begin
        if TimeOut > 0 then
        begin
          // Waiting for MS response
          Sleep ( 100 );
          TimeOut := TimeOut - 1;
        end
        else
        begin
          // Answer wasn't acquired
          Finished := True;
          Result   := False;
        end;
      end;
    until Finished;
  end;



  function  TTNMEComm.Read ( Answers: TMSAnswer ): Boolean;

    function Min ( p1, p2: Integer ): Integer;
    begin
      if p1 = 0 then
        Result := p2
      else if p2 = 0 then
        Result := p1
      else if p1 < p2 then
        Result := p1
      else
        Result := p2;
    end;

  var
    i, p, len : Integer;
    Finished  : Boolean;
    Line      : string;
  begin
    Result := False;
    Answers.Lines.Clear;
    Answers.Status := ME_INVALID;
    repeat
      if Listen then
      begin
        // Separate lines
        p := Min ( Pos ( #10, Input ), Pos ( #13, Input ) );
        len := Length ( Input );
        while 0 < p do
        begin
          Line := Copy ( Input, 1, p - 1 );
          if Line <> '' then
            Answers.Lines.Add ( Line );
          // Skip CRs
          len := Length ( Input );
          while ( p <= len ) and (( Input [p] = #10 ) or ( Input [p] = #13 )) do  p := p + 1;
          Input := Copy ( Input, p, len - p + 1 );
          p := Min ( Pos ( #10, Input ), Pos ( #13, Input ) );
        end;
        // Validate
        if Answers.Count > 0 then
        begin
          if ( Answers.LastLine = 'OK' ) or ( Answers.LastLine = 'ERROR' ) then
          begin
            Result          := True;
            Finished        := True;
            Answers.Status  := ME_VALID;
          end
        end;
      end
      else
        Finished := True;
    until Finished;
  end;



  function   TTNMEComm.Write ( Data: string ): Boolean;
  var
    NumberOfBytesWritten: DWord;
    i:        Integer;
  begin
    Output    := Data + #13;
    OutLength := Length ( Output );
    for i := 1 to OutLength do
      WriteFile ( CommHandle, Output[i], 1, NumberOfBytesWritten, nil );
    Result := True;
  end;



  destructor TTNMEComm.Destroy;
  begin
    if CommHandle <> 0 then
      CloseHandle ( CommHandle );
  end;

end.
